<?php

/**
 * @file
 * Contains \county_handler_filter_string.
 *
 * @author Jim Berry ("solotandem", http://drupal.org/user/240748)
 */

/**
 * Filters on either the county FIPS value or county text.
 *
 * @todo Filter by FIPS value. Might also allow syntax like 'KS:Riley'.
 */
class county_handler_filter_string extends views_handler_filter_string {

  /**
   * Adds filter to the query.
   */
  function query__SAVE() {
    // @todo A join would be simpler if the county_fips table included a column
    // with same contents as the value column in a field table.
    $this->ensure_my_table();
    $field = "$this->table_alias.$this->real_field";
    $this->sub_select($field);
  }

  // Compare filter value to 'county' column in county_fips table.
  function sub_select($field) {
    $value = db_like($this->value);
    $sql = "$field IN (SELECT CONCAT(fips_state, fips_county) FROM county_fips WHERE county LIKE '%$value%')";
    $this->query->add_where_expression($this->options['group'], $sql);
  }



  function option_definition() {
    $options = parent::option_definition();
    $options['value_type'] = array('default' => 'fips');
    return $options;
  }

  /**
   * Form builder.
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['value_type'] = array(
      '#type' => 'radios',
      '#title' => t('Value type'),
      '#description' => t('Indicate whether each filter value is text (Macon County) or FIPS (20161).'),
      '#default_value' => $this->options['value_type'],
      '#options' => ['text' => 'County text', 'fips' => 'FIPS value'],
    );
  }

  /**
   * Returns list of allowed operators.
   */
  function operators() {
    $operators = parent::operators();
    unset($operators['word'], $operators['allwords'], $operators['shorterthan'], $operators['longerthan'], $operators['regular_expression']);
    return $operators;
  }

  /**
   * Adds filter to the query.
   */
  function query() {
    if ($this->options['value_type'] == 'fips') {
      parent::query();
      return;
    }

    $this->ensure_my_table();
    $field = "$this->table_alias.$this->real_field";

    // Join the county field table with the county_fips table.
    // This needs a custom expression that uses substr of values
    // Add expression where filter value matches the 'county' column in county_fips table.
    $join = new views_join();
    $join->definition = array(
      'left_table' => $this->table_alias,
      'left_field' => $this->real_field,

//       'left_table' => 'county_fips',
//       'left_field' => 'county',

      'left_table' => '',
      'left_field' => "SUBSTR($field, 1, 2)",

      'table'      => 'county_fips',
      'field'      => 'fips_state', // 'county',
      'type'       => 'LEFT', // 'INNER',
      'extra'      => array(
//         array(
//           'field' => 'fips_state',
//           'value' => substr($this->value, 0, 2), // [exposed] filter value chars 1-2
//         ),
        array(
          'field' => 'fips_county',
//           'value' => "SUBSTR($field, 2, 3)", // substr($this->value, 2, 3), // [exposed] filter value chars 3-5
          'formula'    => "SUBSTR($field, 3, 3)",
        ),
//         array(
//           'formula'    => $field . ' = CONCAT(county_fips.fips_state, county_fips.fips_county)',
//         ),
      ),
    );
/*
    $join->definition = array(
      'formula'    => 'CONCAT(county_fips.fips_state, county_fips.fips_county)',
      'table' => $this->table_alias,
      'field' => $this->real_field,
      'type'       => 'INNER',
    );
    $join->definition = array(
      'formula'    => $field . ' = CONCAT(county_fips.fips_state, county_fips.fips_county)',
      'type'       => 'INNER',
    );
*/
    $join->construct();
    $join->adjusted = TRUE;
//     $fips = $this->query->ensure_table('county_fips', $this->relationship, $join);
    $fips = $this->query->queue_table('county_fips', $this->relationship, $join/*, $alias*/);

    $info = $this->operators();

    // @todo $value seems unnecessary as this is set in parent code.
//     $value = !empty($this->value) ? $this->value : $this->options['value'];
    if (!empty($this->options['break_phrase']) && !in_array($this->options['operator'], ['empty', 'not empty'])) {
      // @todo This should not be called with certain operators that do not depend on the 'value'.
      // For example, empty or not empty.
      // The operator is changed to 'or' or 'and in next routine.
      views_break_phrase_string($this->value, $this);
    }
    else {
      // Restore the operator.
      // The operator is changed to 'or' in ::title() routine.
      // This seems to be a namespace clash between the 'or' operator for an
      // argument handler and the usual 'operator' for a filter handler. The
      // former is a 'multiple value' operator used with 'break_phrase'.
      $this->operator = $this->options['operator'];
    }

    if (count($this->value) > 1) {
      $this->operator = 'IN';
      $info[$this->operator]['method'] = 'op_multiple';
    }

    if (!empty($info[$this->operator]['method'])) {
      if ($info[$this->operator]['method'] != 'op_empty') {
        // Conditions involving NULL need to be applied against real_field; all
        // others against the fips county value.
        $field = "$fips.county";
      }
      $this->{$info[$this->operator]['method']}($field);
    }
  }

  // Each of these should compare filter value to 'county' column in county_fips table.
  // Use the relationship from the query above.
  function op_multiple($field) {
    $this->query->add_where($this->options['group'], $field, $this->value, $this->operator);
  }
}
